#Writen by Nicolas Mongiardino Koch during 2020 as supplementary material of
#Mongiardino Koch (2021) - Phylogenomic subsampling and the search for
#phylogenetically reliable loci. bioRxiv 2021.02.13.431075.
#https://doi.org/10.1101/2021.02.13.431075

#This iterates through the datasets and saves the alignments that are used to
#estimate gene trees with ParGenes (Morel, et al. 2018,
#https://github.com/BenoitMorel/ParGenes).

library(phangorn)
library(tidyverse)

#Change working directory to folder 'Datasets' containing all data files
setwd('')
rm(list=ls())
folders = substring(list.dirs(recursive = F), 2)

for(n in 1:length(folders)) {
  #read matrix and partitions
  data = read.phyDat(paste0(getwd(), folders[n], '/supermatrix_red.fa'), format = 'fasta', type = 'AA')
  data = as.AAbin(data)
  partitions = read.table(paste0(getwd(), folders[n], '/partitions_red.txt'), sep = ' ')
  partitions = as.tibble(partitions[,4])
  partitions = partitions %>% separate(value, into = c('Start', 'End'), sep = '-') %>%
    mutate_if(is.character, as.numeric)

  #create subdirectory
  dir.create(paste0(getwd(), folders[n], folders[n]))

  #save alignments after removing taxa with all missing data
  for(i in 1:nrow(partitions)) {
    gene = data[,partitions$Start[i]:partitions$End[i]]
    gene = as.character(gene)

    delete_taxa = which(apply(gene, 1, function(x) all(x == '-')))
    if(length(delete_taxa) > 0) {
      gene = gene[-delete_taxa,]
    }

    write.phyDat(as.phyDat(as.AAbin(gene)), file = paste0(paste0(getwd(), folders[n], folders[n]), '/gene', i, '.fa'), format = 'fasta')
  }
}
