#Writen by Nicolas Mongiardino Koch during 2020 as supplementary material of
#Mongiardino Koch (2021) - Phylogenomic subsampling and the search for
#phylogenetically reliable loci. bioRxiv 2021.02.13.431075.
#https://doi.org/10.1101/2021.02.13.431075

#This iterates through the datasets and places all gene trees into a single file
#named 'all_gene_trees.tre' (provided as data file) as well as a txt file
#containing the names of those loci which failed to run, named
#'missing_gene_trees.txt' (also provided).

library(ape)
library(readr)

#Change working directory to folder 'Datasets' containing all data files
setwd('')
rm(list=ls())
`%not in%` <- function (x, table) is.na(match(x, table, nomatch=NA_integer_))
folders = substring(list.dirs(recursive = F), 2)

for(n in 1:length(folders)) {
  #if gene trees have been estimated
  if(grepl('result', list.dirs(path = paste0(getwd(), folders[n]), recursive = F))) {
    #read partitions to know the number of genes to expect
    partitions = read.table(paste0(getwd(), folders[n], '/partitions_red_noOG.txt'), sep = ' ')
    num_trees = nrow(partitions)
    
    #read trees and order by number
    trees = list.files(path = paste0(getwd(), folders[n], '/gene_trees'), pattern = 'raxml.support')
    trees = trees[order(parse_number(trees))]
    
    #find missing ones
    missing = which(1:num_trees %not in% parse_number(trees))
    
    #put all trees into a single file
    for(i in 1:length(trees)) {
      if(i == 1) {
        multi_trees = list(read.tree(paste0(getwd(), folders[n], '/gene_trees/', trees[i])))
        class(multi_trees) = "multiPhylo"
      } else {
        this_tree = list(read.tree(paste0(getwd(), folders[n], '/gene_trees/', trees[i])))
        class(this_tree) = "multiPhylo"
        multi_trees = c(multi_trees, this_tree)
      }
    }
    
    #write all trees to file, as well as a text file specifying which failed at
    #the inference step
    write.tree(multi_trees, file = paste0(getwd(), folders[n], '/all_gene_trees.tre'))
    if(length(missing) > 0) {
      write.table(missing, file = paste0(getwd(), folders[n], '/missing_gene_trees.txt'),
                  quote = F, row.names = F, col.names = F, sep = '\n')
    }
  }
}
